/**
 * Licensed to Abiquo Holdings S.L. (Abiquo) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package com.abiquo.server.core.cloud;

import javax.validation.constraints.NotNull;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import org.apache.commons.lang.StringUtils;

import com.abiquo.model.doc.Desc;
import com.abiquo.model.doc.Output;
import com.abiquo.model.transport.SingleResourceTransportDto;
import com.abiquo.server.core.cloud.chef.RunlistElementsDto;
import com.abiquo.server.core.infrastructure.storage.DvdManagementDto;

@XmlRootElement(name = "virtualMachine")
public class VirtualMachineDto extends SingleResourceTransportDto implements
    Comparable<VirtualMachineDto>
{
    /**
     * 
     */
    private static final long serialVersionUID = -8877350185009627544L;

    public static final String BASE_MEDIA_TYPE = "application/vnd.abiquo.virtualmachine+xml";

    public static final String MEDIA_TYPE = BASE_MEDIA_TYPE + "; version=" + API_VERSION;

    public static final String MEDIA_TYPE_20 = BASE_MEDIA_TYPE + "; version=" + API_VERSION_20;

    private Integer id;

    private String name;

    private String description;

    private int ram;

    private int cpu;

    private long hdInBytes;

    private int vdrpPort;

    private String vdrpIP;

    private int idState;

    private VirtualMachineState state;

    private int highDisponibility;

    private int idType;

    private String password;

    private String keymap;

    private String uuid;

    private RunlistElementsDto runlistElements;

    private DvdManagementDto dvd;

    @Override
    public int compareTo(final VirtualMachineDto dto)
    {
        VirtualMachineDto vm2 = dto;
        if (StringUtils.isNotBlank(this.getName()) && StringUtils.isNotBlank(vm2.getName()))
        {
            return this.getName().compareTo(vm2.getName());
        }
        else if (!StringUtils.isNotBlank(this.getName()) && !StringUtils.isNotBlank(vm2.getName()))
        {
            return 0;
        }
        else if (!StringUtils.isNotBlank(this.getName()) && StringUtils.isNotBlank(vm2.getName()))
        {
            return -1;
        }
        else
        {
            return 1;
        }
    }

    @Override
    public String getBaseMediaType()
    {
        return BASE_MEDIA_TYPE;
    }

    @Desc("Number of CPUs of the virtual machine")
    @NotNull
    public int getCpu()
    {
        return cpu;
    }

    public String getDescription()
    {
        return description;
    }

    @Desc("Whether the virtual machine must mount a CD or DVD drive")
    @XmlElement(name = "dvd")
    public DvdManagementDto getDvd()
    {
        return dvd;
    }

    @Desc("Hard disk size in Bytes")
    @NotNull
    public long getHdInBytes()
    {
        return hdInBytes;
    }

    @Desc("Deprecated")
    @NotNull
    public int getHighDisponibility()
    {
        return highDisponibility;
    }

    @Desc("Opaque ID of the virtual machine")
    @Output
    public Integer getId()
    {
        return id;
    }

    @Desc("Opaque ID of the state")
    public int getIdState()
    {
        return idState;
    }

    @Desc("Opaque ID of the virtual machine type")
    public int getIdType()
    {
        return idType;
    }

    @Desc("Keyboard keymap to be setted on the virtual machine")
    public String getKeymap()
    {
        return keymap;
    }

    @Override
    public String getMediaType()
    {
        return VirtualMachineDto.MEDIA_TYPE;
    }

    @Desc("Name of the virtual machine")
    @NotNull
    public String getName()
    {
        return name;
    }

    @Desc("VNC password")
    public String getPassword()
    {
        return password;
    }

    @Desc("Ram size of the virtual machine")
    @NotNull
    public int getRam()
    {
        return ram;
    }

    @Desc("Chef runlist to bootstrap")
    public RunlistElementsDto getRunlist()
    {
        return runlistElements;
    }

    @Desc("Current virtual machine state")
    public VirtualMachineState getState()
    {
        return state;
    }

    @Desc("UUID of the virtual machine")
    @Output
    public String getUuid()
    {
        return uuid;
    }

    @Desc("Virtual remote desktop IP")
    @Output
    public String getVdrpIP()
    {
        return vdrpIP;
    }

    @Desc("Virtual remote desktop port")
    @Output
    public int getVdrpPort()
    {
        return vdrpPort;
    }

    public void setCpu(final int cpu)
    {
        this.cpu = cpu;
    }

    public void setDescription(final String description)
    {
        this.description = description;
    }

    public void setDvd(final DvdManagementDto dvd)
    {
        this.dvd = dvd;
    }

    public void setHdInBytes(final long hdInBytes)
    {
        this.hdInBytes = hdInBytes;
    }

    public void setHighDisponibility(final int highDisponibility)
    {
        this.highDisponibility = highDisponibility;
    }

    public void setId(final Integer id)
    {
        this.id = id;
    }

    public void setIdState(final int idState)
    {
        this.idState = idState;
    }

    public void setIdType(final int idType)
    {
        this.idType = idType;
    }

    public void setKeymap(final String keymap)
    {
        this.keymap = keymap;
    }

    public void setName(final String name)
    {
        this.name = name;
    }

    public void setPassword(final String password)
    {
        this.password = password;
    }

    public void setRam(final int ram)
    {
        this.ram = ram;
    }

    public void setRunlist(final RunlistElementsDto runlistElements)
    {
        this.runlistElements = runlistElements;
    }

    public void setState(final VirtualMachineState state)
    {
        this.state = state;
    }

    public void setUuid(final String uuid)
    {
        this.uuid = uuid;
    }

    public void setVdrpIP(final String vdrpIP)
    {
        this.vdrpIP = vdrpIP;
    }

    public void setVdrpPort(final int vdrpPort)
    {
        this.vdrpPort = vdrpPort;
    }

}
