/**
 * Licensed to Abiquo Holdings S.L. (Abiquo) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package com.abiquo.server.core.config;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.validation.constraints.NotNull;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.XmlType;

import com.abiquo.model.doc.Desc;
import com.abiquo.model.doc.Output;
import com.abiquo.model.transport.SingleResourceTransportDto;

@XmlRootElement(name = "license")
@XmlType(propOrder = {"id", "customerid", "enabledip", "numcores", "expiration", "code"})
public class LicenseDto extends SingleResourceTransportDto
{
    private static final long serialVersionUID = 1L;

    public static final String BASE_MEDIA_TYPE = "application/vnd.abiquo.license+xml";

    public static final String MEDIA_TYPE = BASE_MEDIA_TYPE + "; version=" + API_VERSION;

    /** The date format used to store license dates. */
    public static final String DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";

    /** The license id **/
    private Integer id;

    /** The license code. */
    private String code;

    /** The customer ID. */
    private String customerid;

    /** The enabled IP. */
    // Not used yet. Do not serialize to xml.
    @XmlTransient
    private String enabledip;

    /** The number of cores. */
    private Integer numcores;

    /** The expiration date. */
    private String expiration;

    @Desc("Identifier of the entity")
    public Integer getId()
    {
        return id;
    }

    public void setId(final Integer id)
    {
        this.id = id;
    }

    @NotNull
    @Desc("The license code")
    public String getCode()
    {
        return code;
    }

    public void setCode(final String code)
    {
        this.code = code;
    }

    @Output
    @Desc("The customer id for the license owner")
    public String getCustomerid()
    {
        return customerid;
    }

    public void setCustomerid(final String customerid)
    {
        this.customerid = customerid;
    }

    @Output
    @Desc("The licensed ip address")
    public String getEnabledip()
    {
        return enabledip;
    }

    public void setEnabledip(final String enabledip)
    {
        this.enabledip = enabledip;
    }

    @Output
    @Desc("The number of cores that the license enables")
    public Integer getNumcores()
    {
        return numcores;
    }

    public void setNumcores(final Integer numcores)
    {
        this.numcores = numcores;
    }

    @Output
    @Desc("The expiration date of the license")
    public String getExpiration()
    {
        return expiration;
    }

    public void setExpiration(final String expiration)
    {
        this.expiration = expiration;
    }

    @Override
    public String getMediaType()
    {
        return LicenseDto.MEDIA_TYPE;
    }

    @Override
    public String getBaseMediaType()
    {
        return BASE_MEDIA_TYPE;
    }

    // *************************** Helper methods ***********************

    @XmlTransient
    public boolean isActive()
    {
        SimpleDateFormat format = new SimpleDateFormat(DATE_FORMAT);
        try
        {
            Date expiration = format.parse(getExpiration());
            return expiration.getTime() >= System.currentTimeMillis();
        }
        catch (ParseException e)
        {
            throw new RuntimeException("The license expiration date cannot be parsed");
        }
    }
}
