/**
 * Licensed to Abiquo Holdings S.L. (Abiquo) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package com.abiquo.server.core.enterprise;

import javax.validation.constraints.NotNull;
import javax.xml.bind.annotation.XmlRootElement;

import com.abiquo.model.doc.Desc;
import com.abiquo.model.doc.Output;
import com.abiquo.model.transport.SingleResourceTransportDto;

@XmlRootElement(name = "user")
public class UserDto extends SingleResourceTransportDto
{
    /**
     * UID.
     */
    private static final long serialVersionUID = -3233121860012587479L;

    public static final String BASE_MEDIA_TYPE = "application/vnd.abiquo.user+xml";

    public static final String MEDIA_TYPE = BASE_MEDIA_TYPE + "; version=" + API_VERSION;

    private Integer id;

    public UserDto()
    {

    }

    /**
     * Constructor.
     * 
     * @param name Name.
     * @param surname Surname.
     * @param email Email Address.
     * @param nick Login.
     * @param password Password.
     * @param locale Language.
     * @param description Desc.
     * @deprecated use
     *             {@link #UserDto(String, String, String, String, String, String, String, String)}
     *             instead.
     */
    @Deprecated
    public UserDto(final String name, final String surname, final String email, final String nick,
        final String password, final String locale, final String description)
    {
        this.name = name;
        this.surname = surname;
        this.email = email;
        this.nick = nick;
        this.password = password;
        this.locale = locale;
        this.description = description;
    }

    /**
     * Constructor.
     * 
     * @param name Name.
     * @param surname Surname.
     * @param email Email Address.
     * @param nick Login.
     * @param password Password.
     * @param locale Language.
     * @param description Desc.
     * @param authType AuthType value.
     */
    public UserDto(final String name, final String surname, final String email, final String nick,
        final String password, final String locale, final String description, final String authType)
    {
        this.name = name;
        this.surname = surname;
        this.email = email;
        this.nick = nick;
        this.password = password;
        this.locale = locale;
        this.description = description;
        this.authType = authType;
    }

    @Output    
    @Desc("Identifier for this user")
    public Integer getId()
    {
        return id;
    }

    public void setId(final Integer id)
    {
        this.id = id;
    }

    private String name;

    @NotNull
    @Desc("User's real first name")
    public String getName()
    {
        return name;
    }

    public void setName(final String name)
    {
        this.name = name;
    }

    private String nick;

    @NotNull
    @Desc("User name / nickname. The username for login")
    public String getNick()
    {
        return nick;
    }

    public void setNick(final String nick)
    {
        this.nick = nick;
    }

    private String locale;

    @Output
    @Desc("Locale assigned to this user")
    public String getLocale()
    {
        return locale;
    }

    public void setLocale(final String locale)
    {
        this.locale = locale;
    }

    private String password;

    @Desc("Password set for this user")
    public String getPassword()
    {
        return password;
    }

    public void setPassword(final String password)
    {
        this.password = password;
    }

    private String surname;

    @Desc("User's real last name")
    public String getSurname()
    {
        return surname;
    }

    public void setSurname(final String surname)
    {
        this.surname = surname;
    }

    private boolean active;

    @NotNull
    @Desc("Flag indicating if the user is activated or not.")
    public boolean isActive()
    {
        return active;
    }

    public void setActive(final int active)
    {
        this.active = active == 1;
    }

    public void setActive(final boolean active)
    {
        this.active = active;
    }

    private String email;

    @Desc("The contact e-mail address of the user for Abiquo messages")
    public String getEmail()
    {
        return email;
    }

    public void setEmail(final String email)
    {
        this.email = email;
    }

    private String description;

    @Desc("A short description of the user")
    public String getDescription()
    {
        return description;
    }

    public void setDescription(final String description)
    {
        this.description = description;
    }

    // comma separated values
    private String availableVirtualDatacenters;

    @Desc("List with restricted VDCs for this user separated by commas")
    public String getAvailableVirtualDatacenters()
    {
        return availableVirtualDatacenters;
    }

    public void setAvailableVirtualDatacenters(final String availableVirtualDatacenters)
    {
        this.availableVirtualDatacenters = availableVirtualDatacenters;
    }

    /**
     * String representation for {@link com.abiquo.model.enumerator.AuthType} value.
     */
    private String authType;

    /**
     * {@link com.abiquo.model.enumerator.AuthType} value.
     * 
     * @return {@link com.abiquo.model.enumerator.AuthType} value. String.
     */
    @NotNull
    @Desc("Auth type selected for this user: Abiquo managed or LDAP")
    public String getAuthType()
    {
        return authType;
    }

    /**
     * {@link com.abiquo.model.enumerator.AuthType} value.
     * 
     * @param authType {@link com.abiquo.model.enumerator.AuthType} value.
     */
    public void setAuthType(final String authType)
    {
        this.authType = authType;
    }

    @Override
    public String getMediaType()
    {
        return UserDto.MEDIA_TYPE;
    }

    @Override
    public String getBaseMediaType()
    {
        return BASE_MEDIA_TYPE;
    }
}
